﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Interactivity;
using Infragistics.Controls.Editors;
using System.Windows.Data;
using System.Collections;

namespace XamComboEditorAdditionalFeatures
{
    public class CheckBoxSelectionBehavior : Behavior<XamComboEditor>
    {
        private bool _surpressSelectedValuePropertyChangedCallback;
        private bool _surpressSelectedValueUpdate;

        #region Properties

        public static readonly DependencyProperty SelectedMemberPathProperty = DependencyProperty.Register("CheckedMemberPath", typeof(string), typeof(CheckBoxSelectionBehavior), new UIPropertyMetadata(String.Empty));
        public string CheckedMemberPath
        {
            get { return (string)GetValue(SelectedMemberPathProperty); }
            set { SetValue(SelectedMemberPathProperty, value); }
        }

        public static readonly DependencyProperty ItemsSourceProperty = DependencyProperty.Register("ItemsSource", typeof(IEnumerable), typeof(CheckBoxSelectionBehavior), new UIPropertyMetadata(null, new PropertyChangedCallback(OnItemsSourceChanged)));
        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
        }

        private static void OnItemsSourceChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            CheckBoxSelectionBehavior bahavior = o as CheckBoxSelectionBehavior;
            if (bahavior != null)
                bahavior.OnItemsSourceChanged((IEnumerable)e.OldValue, (IEnumerable)e.NewValue);
        }

        private void OnItemsSourceChanged(IEnumerable oldvalue, IEnumerable newValue)
        {
            if (newValue != null)
            {
                _surpressSelectedValuePropertyChangedCallback = true;

                GetInitiallySelectedItems();
                UpdateSelectedValue();

                _surpressSelectedValuePropertyChangedCallback = false;
            }
        }

        public static readonly DependencyProperty SelectedValueProperty = DependencyProperty.Register("SelectedValue", typeof(string), typeof(CheckBoxSelectionBehavior), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, OnSelectedValueChanged));
        public string SelectedValue
        {
            get { return (string)GetValue(SelectedValueProperty); }
            set { SetValue(SelectedValueProperty, value); }
        }

        private static void OnSelectedValueChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            CheckBoxSelectionBehavior bahavior = o as CheckBoxSelectionBehavior;
            if (bahavior != null)
                bahavior.OnSelectedValueChanged((string)e.OldValue, (string)e.NewValue);
        }

        protected virtual void OnSelectedValueChanged(string oldValue, string newValue)
        {
            if (_surpressSelectedValuePropertyChangedCallback)
                return;

            UpdateSelectedItemsFromSelectedValue();
        }

        public static readonly DependencyProperty ValueMemberPathProperty = DependencyProperty.Register("ValueMemberPath", typeof(string), typeof(CheckBoxSelectionBehavior), new UIPropertyMetadata(String.Empty, OnValueMemberPathChanged));
        public string ValueMemberPath
        {
            get { return (string)GetValue(ValueMemberPathProperty); }
            set { SetValue(ValueMemberPathProperty, value); }
        }

        private static void OnValueMemberPathChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            CheckBoxSelectionBehavior bahavior = o as CheckBoxSelectionBehavior;
            if (bahavior != null)
                bahavior.OnValueMemberPathChanged((string)e.OldValue, (string)e.NewValue);
        }

        protected virtual void OnValueMemberPathChanged(string oldValue, string newValue)
        {
            if (!String.IsNullOrWhiteSpace(newValue))
                UpdateSelectedValue();
        }

        #endregion //Properties

        #region Base Class Overrides

        protected override void OnAttached()
        {
            base.OnAttached();

            Binding binding = new Binding("ItemsSource");
            binding.Source = AssociatedObject;
            BindingOperations.SetBinding(this, CheckBoxSelectionBehavior.ItemsSourceProperty, binding);

            AssociatedObject.SelectionChanged += AssociatedObject_SelectionChanged;
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            BindingOperations.ClearBinding(this, CheckBoxSelectionBehavior.ItemsSourceProperty);
            AssociatedObject.SelectionChanged -= AssociatedObject_SelectionChanged;
        }

        #endregion //Base Class Overrides

        #region Event Handlers

        void AssociatedObject_SelectionChanged(object sender, EventArgs e)
        {
            if (_surpressSelectedValuePropertyChangedCallback)
                return;

            //update the selected value string but only if we are not surpressing the operation.
            if (!_surpressSelectedValueUpdate)
                UpdateSelectedValue();

            //because we do not know which item was unchecked, and we cannot create a data binding between the checkbox and the SelectedMemberPath,
            //we have to add this ugly slow hack that will iterate through all the objects and update the SelectedMemberPath of the bound object
            //this may reduce performance on large lists
            foreach (var item in AssociatedObject.Items)
            {
                if (!String.IsNullOrEmpty(CheckedMemberPath))
                {
                    var property = item.Data.GetType().GetProperty(CheckedMemberPath);
                    if (property != null)
                        property.SetValue(item.Data, item.IsSelected, null);
                }
            }
        }

        #endregion //Event Handlers

        #region Methods

        private void ClearSelectedItems()
        {
            if (AssociatedObject != null && AssociatedObject.SelectedItems != null)
                AssociatedObject.SelectedItems.Clear();
        }

        private void GetInitiallySelectedItems()
        {
            if (ItemsSource == null)
                return;

            var list = new List<object>();

            foreach (var item in ItemsSource)
            {
                var property = item.GetType().GetProperty(CheckedMemberPath);
                if (property != null)
                {
                    if ((Boolean)property.GetValue(item, null))
                    {
                        AssociatedObject.SelectedItems.Add(item);
                    }
                }
            }
        }

        protected object ResolveItemByValue(string value)
        {
            if (!String.IsNullOrEmpty(ValueMemberPath))
            {
                if (ItemsSource != null)
                {
                    foreach (object item in ItemsSource)
                    {
                        var property = item.GetType().GetProperty(ValueMemberPath);
                        if (property != null)
                        {
                            var propertyValue = property.GetValue(item, null);
                            if (value.Equals(propertyValue.ToString(), StringComparison.InvariantCultureIgnoreCase))
                                return item;
                        }
                    }
                }
            }

            return value;
        }

        private void UpdateSelectedItemsFromSelectedValue()
        {
            if (AssociatedObject == null)
                return;

            _surpressSelectedValueUpdate = true;

            ClearSelectedItems();

            if (!String.IsNullOrEmpty(SelectedValue))
            {
                string[] values = SelectedValue.Split(new string[] { AssociatedObject.MultiSelectValueDelimiter.ToString() }, StringSplitOptions.RemoveEmptyEntries);
                foreach (string value in values)
                {
                    var item = ResolveItemByValue(value);
                    if (item != null)
                        AssociatedObject.SelectedItems.Add(item);
                }
            }

            _surpressSelectedValueUpdate = false;
        }

        private void UpdateSelectedValue()
        {
            if (AssociatedObject == null)
                return;

            _surpressSelectedValuePropertyChangedCallback = true;

            string newValue = String.Join(AssociatedObject.MultiSelectValueDelimiter.ToString(), AssociatedObject.SelectedItems.Cast<object>().Select(x => GetItemValue(x)));

            if (String.IsNullOrEmpty(SelectedValue) || !SelectedValue.Equals(newValue))
                SelectedValue = newValue;

            _surpressSelectedValuePropertyChangedCallback = false;
        }

        protected object GetItemValue(object item)
        {
            if (!String.IsNullOrEmpty(ValueMemberPath) && (item != null))
            {
                var property = item.GetType().GetProperty(ValueMemberPath);
                if (property != null)
                    return property.GetValue(item, null);
            }

            return item;
        }

        #endregion //Methods
    }
}
